==> heptapod-rails-0.21.3.tar.gz: heptapod-rails-0.21.3/spec/features/projects/services/user_activates_mattermost_slash_command_spec.rb <==
# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Set up Mattermost slash commands', :js do
  describe 'user visits the mattermost slash command config page' do
    include_context 'project service activation'

    before do
      stub_mattermost_setting(enabled: mattermost_enabled)
      visit_project_integration('Mattermost slash commands')
    end

    context 'mattermost service is enabled' do
      let(:mattermost_enabled) { true }

      it 'shows a help message' do
        expect(page).to have_content("This service allows users to perform common")
      end

      it 'shows a token placeholder' do
        token_placeholder = find_field('service_token')['placeholder']

        expect(token_placeholder).to eq('XXxxXXxxXXxxXXxxXXxxXXxx')
      end

      it 'redirects to the integrations page after saving but not activating' do
        token = ('a'..'z').to_a.join

        fill_in 'service_token', with: token
        click_active_checkbox
        click_save_integration

        expect(current_path).to eq(edit_project_service_path(project, :mattermost_slash_commands))
        expect(page).to have_content('Mattermost slash commands settings saved, but not active.')
      end

      it 'redirects to the integrations page after activating' do
        token = ('a'..'z').to_a.join

        fill_in 'service_token', with: token
        click_save_integration

        expect(current_path).to eq(edit_project_service_path(project, :mattermost_slash_commands))
        expect(page).to have_content('Mattermost slash commands settings saved and active.')
      end

      it 'shows the add to mattermost button' do
        expect(page).to have_link('Add to Mattermost')
      end

      it 'shows an explanation if user is a member of no teams' do
        stub_teams(count: 0)

        click_link 'Add to Mattermost'

        expect(page).to have_content('You aren’t a member of any team on the Mattermost instance')
        expect(page).to have_link('join a team', href: "#{Gitlab.config.mattermost.host}/select_team")
      end

      it 'shows an explanation if user is a member of 1 team' do
        stub_teams(count: 1)

        click_link 'Add to Mattermost'

        expect(page).to have_content('The team where the slash commands will be used in')
        expect(page).to have_content('This is the only available team that you are a member of.')
      end

      it 'shows a disabled prefilled select if user is a member of 1 team' do
        teams = stub_teams(count: 1)

        click_link 'Add to Mattermost'

        team_name = teams.first['display_name']
        select_element = find('#mattermost_team_id')
        selected_option = select_element.find('option[selected]')

        expect(select_element['disabled']).to eq("true")
        expect(selected_option).to have_content(team_name.to_s)
      end

      it 'has a hidden input for the prefilled value if user is a member of 1 team' do
        teams = stub_teams(count: 1)

        click_link 'Add to Mattermost'

        expect(find('input#mattermost_team_id', visible: false).value).to eq(teams.first['id'])
      end

      it 'shows an explanation user is a member of multiple teams' do
        stub_teams(count: 2)

        click_link 'Add to Mattermost'

        expect(page).to have_content('Select the team where the slash commands will be used in')
        expect(page).to have_content('The list shows all available teams that you are a member of.')
      end

      it 'shows a select with team options user is a member of multiple teams' do
        stub_teams(count: 2)

        click_link 'Add to Mattermost'

        select_element = find('#mattermost_team_id')

        expect(select_element['disabled']).to be_falsey
        expect(select_element.all('option').count).to eq(3)
      end

      it 'shows an error alert with the error message if there is an error requesting teams' do
        allow_any_instance_of(MattermostSlashCommandsService).to receive(:list_teams) { [[], 'test mattermost error message'] }

        click_link 'Add to Mattermost'

        expect(page).to have_selector('.gl-alert')
        expect(page).to have_content('test mattermost error message')
      end

      it 'enables the submit button if the required fields are provided', :js do
        stub_teams(count: 1)

        click_link 'Add to Mattermost'

        expect(find('input[type="submit"]')['disabled']).not_to eq("true")
      end

      it 'disables the submit button if the required fields are not provided', :js do
        stub_teams(count: 1)

        click_link 'Add to Mattermost'

        fill_in('mattermost_trigger', with: '')

        expect(find('input[type="submit"]')['disabled']).to eq("true")
      end

      def stub_teams(count: 0)
        teams = create_teams(count)

        allow_any_instance_of(MattermostSlashCommandsService).to receive(:list_teams) { [teams, nil] }

        teams
      end

      def create_teams(count = 0)
        teams = []

        count.times do |i|
          teams.push({ "id" => "x#{i}", "display_name" => "x#{i}-name" })
        end

        teams
      end
    end

    context 'mattermost service is not enabled' do
      let(:mattermost_enabled) { false }

      it 'shows the correct trigger url' do
        value = find_field('request_url').value

        expect(value).to match("api/v4/projects/#{project.id}/services/mattermost_slash_commands/trigger")
      end

      it 'shows a token placeholder' do
        token_placeholder = find_field('service_token')['placeholder']

        expect(token_placeholder).to eq('XXxxXXxxXXxxXXxxXXxxXXxx')
      end
    end
  end

  describe 'stable logo url' do
    it 'shows a publicly available logo' do
      expect(File.exist?(Rails.root.join('public/slash-command-logo.png'))).to be_truthy
    end
  end
end

==> heptapod-rails-0.22.0.tar.gz: heptapod-rails-0.22.0/spec/features/projects/services/user_activates_mattermost_slash_command_spec.rb <==
# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Set up Mattermost slash commands', :js do
  describe 'user visits the mattermost slash command config page' do
    include_context 'project service activation'

    before do
      stub_mattermost_setting(enabled: mattermost_enabled)
      visit_project_integration('Mattermost slash commands')
    end

    context 'mattermost service is enabled' do
      let(:mattermost_enabled) { true }

      it 'shows a help message' do
        expect(page).to have_content("Use this service to perform common")
      end

      it 'shows a token placeholder' do
        token_placeholder = find_field('service_token')['placeholder']

        expect(token_placeholder).to eq('XXxxXXxxXXxxXXxxXXxxXXxx')
      end

      it 'redirects to the integrations page after saving but not activating' do
        token = ('a'..'z').to_a.join

        fill_in 'service_token', with: token
        click_active_checkbox
        click_save_integration

        expect(current_path).to eq(edit_project_service_path(project, :mattermost_slash_commands))
        expect(page).to have_content('Mattermost slash commands settings saved, but not active.')
      end

      it 'redirects to the integrations page after activating' do
        token = ('a'..'z').to_a.join

        fill_in 'service_token', with: token
        click_save_integration

        expect(current_path).to eq(edit_project_service_path(project, :mattermost_slash_commands))
        expect(page).to have_content('Mattermost slash commands settings saved and active.')
      end

      it 'shows the add to mattermost button' do
        expect(page).to have_link('Add to Mattermost')
      end

      it 'shows an explanation if user is a member of no teams' do
        stub_teams(count: 0)

        click_link 'Add to Mattermost'

        expect(page).to have_content('You aren’t a member of any team on the Mattermost instance')
        expect(page).to have_link('join a team', href: "#{Gitlab.config.mattermost.host}/select_team")
      end

      it 'shows an explanation if user is a member of 1 team' do
        stub_teams(count: 1)

        click_link 'Add to Mattermost'

        expect(page).to have_content('The team where the slash commands will be used in')
        expect(page).to have_content('This is the only available team that you are a member of.')
      end

      it 'shows a disabled prefilled select if user is a member of 1 team' do
        teams = stub_teams(count: 1)

        click_link 'Add to Mattermost'

        team_name = teams.first['display_name']
        select_element = find('#mattermost_team_id')
        selected_option = select_element.find('option[selected]')

        expect(select_element['disabled']).to eq("true")
        expect(selected_option).to have_content(team_name.to_s)
      end

      it 'has a hidden input for the prefilled value if user is a member of 1 team' do
        teams = stub_teams(count: 1)

        click_link 'Add to Mattermost'

        expect(find('input#mattermost_team_id', visible: false).value).to eq(teams.first['id'])
      end

      it 'shows an explanation user is a member of multiple teams' do
        stub_teams(count: 2)

        click_link 'Add to Mattermost'

        expect(page).to have_content('Select the team where the slash commands will be used in')
        expect(page).to have_content('The list shows all available teams that you are a member of.')
      end

      it 'shows a select with team options user is a member of multiple teams' do
        stub_teams(count: 2)

        click_link 'Add to Mattermost'

        select_element = find('#mattermost_team_id')

        expect(select_element['disabled']).to be_falsey
        expect(select_element.all('option').count).to eq(3)
      end

      it 'shows an error alert with the error message if there is an error requesting teams' do
        allow_any_instance_of(MattermostSlashCommandsService).to receive(:list_teams) { [[], 'test mattermost error message'] }

        click_link 'Add to Mattermost'

        expect(page).to have_selector('.gl-alert')
        expect(page).to have_content('test mattermost error message')
      end

      it 'enables the submit button if the required fields are provided', :js do
        stub_teams(count: 1)

        click_link 'Add to Mattermost'

        expect(find('input[type="submit"]')['disabled']).not_to eq("true")
      end

      it 'disables the submit button if the required fields are not provided', :js do
        stub_teams(count: 1)

        click_link 'Add to Mattermost'

        fill_in('mattermost_trigger', with: '')

        expect(find('input[type="submit"]')['disabled']).to eq("true")
      end

      def stub_teams(count: 0)
        teams = create_teams(count)

        allow_any_instance_of(MattermostSlashCommandsService).to receive(:list_teams) { [teams, nil] }

        teams
      end

      def create_teams(count = 0)
        teams = []

        count.times do |i|
          teams.push({ "id" => "x#{i}", "display_name" => "x#{i}-name" })
        end

        teams
      end
    end

    context 'mattermost service is not enabled' do
      let(:mattermost_enabled) { false }

      it 'shows the correct trigger url' do
        value = find_field('request_url').value

        expect(value).to match("api/v4/projects/#{project.id}/services/mattermost_slash_commands/trigger")
      end

      it 'shows a token placeholder' do
        token_placeholder = find_field('service_token')['placeholder']

        expect(token_placeholder).to eq('XXxxXXxxXXxxXXxxXXxxXXxx')
      end
    end
  end

  describe 'stable logo url' do
    it 'shows a publicly available logo' do
      expect(File.exist?(Rails.root.join('public/slash-command-logo.png'))).to be_truthy
    end
  end
end
